import * as crypto from '@shardeum-foundation/lib-crypto-utils';
import { AxiosRequestConfig } from 'axios';
import { Archiver } from './types';
export declare const getArchiverList: (opts?: {
    customConfigPath?: string;
    customArchiverListEnv?: string;
    archiverTimeoutInMilliSeconds?: number;
    customArchiverList?: Archiver[];
}) => Promise<Archiver[]>;
/**
 * Sets up the Archiver Discovery component.
 * This function throws an exception if it could not find any archiver across different config sources (or)
 * if no archivers responded when fetching the active archiver list.
 * @async
 * @param {object} opts - The options for setting up Archiver Discovery.
 * @param {string} opts.hashKey - The hash key for crypto utils initialisation.
 * @param {boolean} [opts.disableGlobalArchiverList] - Whether to disable setting the global archiver list (use it with caution).
 * @param {string} [opts.customConfigPath] - The path to the custom config file.
 * @param {string} [opts.customArchiverListEnv] - The name of the custom environment variable for archiver list.
 * @param {number} [opts.archiverTimeoutInMilliSeconds] - The timeout for archiver request in milliseconds.
 */
export declare const setupArchiverDiscovery: (opts: {
    hashKey?: string;
    disableGlobalArchiverList?: boolean;
    customConfigPath?: string;
    customArchiverListEnv?: string;
    archiverTimeoutInMilliSeconds?: number;
    customArchiverList?: Archiver[];
}) => Promise<void>;
/**
 * Sends a GET request to an Archiver endpoint and returns the response as a verified SignedObject.
 * @async
 * @template ResponseType - The type of the response data, which should extend crypto.SignedObject.
 * @param {string} endpoint - The Archiver endpoint to query. This should not include a preceding '/'.
 * @param {AxiosRequestConfig} [config] - Optional Axios request configuration.
 * @returns {Promise<ResponseType | null>} A promise that resolves with the response data as a SignedObject, or null if an error occurred.
 */
export declare const getFromArchiver: <ResponseType extends crypto.SignedObject>(endpoint: string, config?: AxiosRequestConfig) => Promise<ResponseType>;
/**
 * Returns the list of archivers that were discovered after setupArchiverDiscovery.
 *
 * @returns {Archiver[]} The list of archivers.
 */
export declare const getFinalArchiverList: () => Archiver[];
