"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFinalArchiverList = exports.getFromArchiver = exports.setupArchiverDiscovery = exports.getArchiverList = void 0;
const crypto = __importStar(require("@shardeum-foundation/lib-crypto-utils"));
const helpers_1 = require("./helpers");
const sources_1 = require("./sources");
const utils_1 = require("./utils");
let finalArchiverList = [];
const getArchiverList = (opts) => __awaiter(void 0, void 0, void 0, function* () {
    const config = yield (0, helpers_1.readConfigFromFile)({ customConfigPath: opts === null || opts === void 0 ? void 0 : opts.customConfigPath });
    const archiverListFromEnv = (0, sources_1.fetchArchiverListFromEnv)({ customEnvName: opts === null || opts === void 0 ? void 0 : opts.customArchiverListEnv });
    const archiverListFromConfig = (0, sources_1.fetchArchiverListFromConfig)(config);
    const archiverListFromRemote = yield (0, sources_1.fetchArchiverListFromRemoteOrCache)(config);
    const archiverListFromCustomList = (opts === null || opts === void 0 ? void 0 : opts.customArchiverList) || [];
    (0, utils_1.shuffleList)(archiverListFromEnv);
    (0, utils_1.shuffleList)(archiverListFromConfig);
    (0, utils_1.shuffleList)(archiverListFromRemote);
    (0, utils_1.shuffleList)(archiverListFromCustomList);
    let combinedArchiverList = [
        ...archiverListFromEnv,
        ...archiverListFromConfig,
        ...archiverListFromRemote,
        ...archiverListFromCustomList,
    ];
    combinedArchiverList = (0, helpers_1.sanitizeArchiverList)(combinedArchiverList);
    if (combinedArchiverList.length === 0) {
        console.log("Couldn't find any archiver");
        throw new Error("Couldn't find any archiver");
    }
    finalArchiverList = (0, helpers_1.removeDuplicateArchiversByPubKey)(combinedArchiverList);
    const currentArchiverList = yield (0, exports.getFromArchiver)('archivers', {
        timeout: opts === null || opts === void 0 ? void 0 : opts.archiverTimeoutInMilliSeconds,
    });
    if (!currentArchiverList) {
        console.log('No archivers responded when fetching current active archivers');
        throw new Error('No archivers responded');
    }
    finalArchiverList.push(...currentArchiverList.activeArchivers);
    return (0, helpers_1.removeDuplicateArchiversByPubKey)(finalArchiverList);
});
exports.getArchiverList = getArchiverList;
/**
 * Sets up the Archiver Discovery component.
 * This function throws an exception if it could not find any archiver across different config sources (or)
 * if no archivers responded when fetching the active archiver list.
 * @async
 * @param {object} opts - The options for setting up Archiver Discovery.
 * @param {string} opts.hashKey - The hash key for crypto utils initialisation.
 * @param {boolean} [opts.disableGlobalArchiverList] - Whether to disable setting the global archiver list (use it with caution).
 * @param {string} [opts.customConfigPath] - The path to the custom config file.
 * @param {string} [opts.customArchiverListEnv] - The name of the custom environment variable for archiver list.
 * @param {number} [opts.archiverTimeoutInMilliSeconds] - The timeout for archiver request in milliseconds.
 */
const setupArchiverDiscovery = (opts) => __awaiter(void 0, void 0, void 0, function* () {
    // init crypto utils
    crypto.init(opts.hashKey ? opts.hashKey : '69fa4195670576c0160d660c3be36556ff8d504725be8a59b5a96509e0c994bc');
    if (!opts.disableGlobalArchiverList) {
        // init active archiver list
        finalArchiverList = yield (0, exports.getArchiverList)({
            customConfigPath: opts.customConfigPath,
            customArchiverListEnv: opts.customArchiverListEnv,
            archiverTimeoutInMilliSeconds: opts.archiverTimeoutInMilliSeconds,
            customArchiverList: opts.customArchiverList,
        });
    }
});
exports.setupArchiverDiscovery = setupArchiverDiscovery;
/**
 * Sends a GET request to an Archiver endpoint and returns the response as a verified SignedObject.
 * @async
 * @template ResponseType - The type of the response data, which should extend crypto.SignedObject.
 * @param {string} endpoint - The Archiver endpoint to query. This should not include a preceding '/'.
 * @param {AxiosRequestConfig} [config] - Optional Axios request configuration.
 * @returns {Promise<ResponseType | null>} A promise that resolves with the response data as a SignedObject, or null if an error occurred.
 */
const getFromArchiver = (endpoint, config) => __awaiter(void 0, void 0, void 0, function* () {
    for (const archiver of finalArchiverList) {
        try {
            const url = `http://${archiver.ip}:${archiver.port}/${endpoint}`;
            const response = yield (0, utils_1.axiosGet)(url, config);
            if (response.status < 200 || response.status >= 500 || !response.data) {
                console.log(`Failed to fetch data from archiver ${archiver.ip}:${archiver.port}`);
                continue;
            }
            if (!crypto.verifyObj(response.data)) {
                console.log(`Invalid signature when fetching from archiver ${archiver.ip}:${archiver.port}`);
                continue;
            }
            finalArchiverList.unshift(archiver);
            finalArchiverList = (0, helpers_1.removeDuplicateArchiversByPubKey)(finalArchiverList);
            return response.data;
        }
        catch (err) {
            console.log(`Failed to fetch data from archiver ${archiver.ip}:${archiver.port}: ${err}`);
        }
    }
    return null;
});
exports.getFromArchiver = getFromArchiver;
/**
 * Returns the list of archivers that were discovered after setupArchiverDiscovery.
 *
 * @returns {Archiver[]} The list of archivers.
 */
const getFinalArchiverList = () => finalArchiverList;
exports.getFinalArchiverList = getFinalArchiverList;
//# sourceMappingURL=index.js.map