"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchArchiverListFromRemoteOrCache = exports.fetchArchiverListFromConfig = exports.fetchArchiverListFromEnv = void 0;
const fs = __importStar(require("fs"));
const utils_1 = require("./utils");
const projectFlags_1 = require("./config/projectFlags");
const fetchArchiverListFromEnv = (opts) => {
    const envName = (opts === null || opts === void 0 ? void 0 : opts.customEnvName) || 'ARCHIVER_INFO';
    // eslint-disable-next-line security/detect-object-injection
    const archiverEnv = process.env[envName];
    if (!archiverEnv) {
        console.log(`Environment variable ${envName} is not defined`);
        return [];
    }
    if (archiverEnv.length === 0) {
        console.log(`Environment variable ${envName} is empty`);
        return [];
    }
    const archivers = archiverEnv.split(',').map((archiver) => archiver.trim());
    if (archivers.length === 0) {
        console.log(`Environment variable ${envName} contains no archivers`);
        return [];
    }
    return archivers.map((archiver) => {
        const [ip, port, publicKey] = archiver.split(':');
        const archiverObj = { ip, port: parseInt(port, 10), publicKey };
        return archiverObj;
    });
};
exports.fetchArchiverListFromEnv = fetchArchiverListFromEnv;
const fetchArchiverListFromConfig = (config) => {
    if (!config.archivers || config.archivers.length === 0) {
        console.log(`Archivers not found in config`);
        return [];
    }
    return config.archivers;
};
exports.fetchArchiverListFromConfig = fetchArchiverListFromConfig;
const readArchiverListFromLocalCache = (filePath) => __awaiter(void 0, void 0, void 0, function* () {
    let jsonData = '';
    try {
        // eslint-disable-next-line security/detect-non-literal-fs-filename
        jsonData = fs.readFileSync(filePath, 'utf-8');
    }
    catch (err) {
        console.log(`Unable to read from local config file: ${filePath} err: ${err}}`);
        return [];
    }
    const parsedData = JSON.parse(jsonData);
    const archiverList = parsedData;
    return archiverList;
});
const fetchArchiverListFromRemoteOrCache = (config, opts) => __awaiter(void 0, void 0, void 0, function* () {
    if (!config.archiversUrl) {
        console.log(`Archivers URL not found in config`);
        return [];
    }
    const cacheFilePath = (opts === null || opts === void 0 ? void 0 : opts.customCacheFilePath) || projectFlags_1.projectFlags.ARCHIVER_SEED_LIST_PATH;
    yield (0, utils_1.downloadAndSaveJsonFile)(config.archiversUrl, cacheFilePath);
    return readArchiverListFromLocalCache(cacheFilePath);
});
exports.fetchArchiverListFromRemoteOrCache = fetchArchiverListFromRemoteOrCache;
//# sourceMappingURL=sources.js.map